#ifndef __RBTREE_H__
#define __RBTREE_H__

#include <cstdlib>
#include <vector>
#include <stack>
#include <functional>
#include <cassert>

enum Color { R, B };

/**
 * @brief Node type of a balanced binary search tree(RB tree).
 * 
 * @tparam _Key  Type of key objects.
 * @tparam _Value  Type of mapped objects.
 */
template <typename _Key, typename _Value>
class RBNode
{
public:
	RBNode() : key(), value(), left(NULL), right(NULL), color(Color::B) {}
	RBNode(_Key k, _Value v, RBNode<_Key, _Value> *NIL) : key(k), value(v), left(NIL), right(NIL), color(Color::R) {}

	_Key key;     ///< key of this node.
	_Value value; ///< value of this node.
	
	RBNode *left;  ///< pointing to the left child node of this node.
	RBNode *right; ///< pointing to the right child node of this node.

	Color color; ///< color.
};

/**
 * @brief A balanced binary search tree(RB tree), which can be retrieved based on a key, in logarithmic time.
 * 
 * @tparam _Key  Type of key objects.
 * @tparam _Value  Type of mapped objects.
 * @tparam _Compare  Comparison function object type, defaults to less<_Key>.
 */
template <typename _Key, typename _Value, typename _Compare = std::less<_Key> >
class RBTree
{
public:
	/** @name constructors, destructor of RBTree. */
	///@{
	RBTree() : node_count(0), key_compare(), root(NULL) { NIL = new RBNode<_Key, _Value>; }
	RBTree(const _Compare& _compare) : node_count(0), key_compare(), root(NULL) { NIL = new RBNode<_Key, _Value>; }
	RBTree(const RBTree& rhs) : node_count(rhs.node_count), key_compare(rhs.key_compare), root(rhs.root), NIL(rhs.NIL) {}
	~RBTree() { clear(); delete NIL; }
	///@}

	RBTree& operator=(const RBTree& rhs)
	{
		node_count = rhs.node_count;
		key_compare = rhs.key_compare;
		root = rhs.root;
		NIL = rhs.NIL;
		return *this;
	}

	/** @name APIs of RBTree. */
	///@{
	/** @brief return true if RBTree has no nodes. */
	bool empty() { return node_count == 0; }
	/** @brief return the number of nodes of RBTree. */
	size_t size() { return node_count; }
	/** @brief return the left-most node of RBTree. */
	RBNode<_Key, _Value>* minimum();
	/** @brief return the right-most node of RBTree. */
	RBNode<_Key, _Value>* maximum();
	/** @brief return the point to the finding node if found, return NULL if not found. */
	RBNode<_Key, _Value>* find(_Key k);
	/**
	 * @brief insert a node.
	 * 
	 * @param k key of RBNode.
	 * @param v value of RBNode.
	 * @return point to the inserted RBNode.
	 */
	RBNode<_Key, _Value>* insert(_Key k, _Value v);
	/** @brief erase a node only need adjust points. */
	bool erase(_Key k);
	/** @brief erase all the nodes of RBTree. */
	void clear();
	///@}

	/** @name Traverse methods of RBTree. */
	///@{
	void preOrderTraverse(std::vector<RBNode<_Key, _Value>* >& vec);
	void inOrderTraverse(std::vector<RBNode<_Key, _Value>* >& vec);
	void postOrderTraverse(std::vector<RBNode<_Key, _Value>* >& vec);
	///@}

	/** @name Debug methods of RBTree. */
	///@{
	void validateBlackHeight();
	void fetchBlackHeight(RBNode<_Key, _Value> *node, int& ph, int& h);
	///@}

private:
	/** @name Critical methods of RBTree. */
	///@{
	/** @brief rebalance after inserting a node. */
	void _insertRebalance(RBNode<_Key, _Value> *p, std::vector<RBNode<_Key, _Value>* >& S);
	/** @brief rebalance after erasing a node. */
	void _eraseRebalance(RBNode<_Key, _Value> *p, std::vector<RBNode<_Key, _Value>* >& S);

	/** @brief execute left rotate as to node p. */
	RBNode<_Key, _Value>* leftRotate(RBNode<_Key, _Value> **p);
	/** @brief execute right rotate as to node p. */
	RBNode<_Key, _Value>* rightRotate(RBNode<_Key, _Value> **p);
	///@}

private:
	size_t node_count;          ///< maintaining the number of nodes of RBTree.
	_Compare key_compare;       ///< key compare predicate of RBNode.
	RBNode<_Key, _Value> *root; ///< root node of RBTree.
	RBNode<_Key, _Value> *NIL;  ///< nil node of RBTree.
};

//////////////////////////////////////////////////////////////////////////
////////////////////          Implementation          ////////////////////
//////////////////////////////////////////////////////////////////////////
template <typename _Key, typename _Value, typename _Compare>
RBNode<_Key, _Value>* RBTree<_Key, _Value, _Compare>::minimum()
{
	if (root == NULL) return NULL;

	RBNode<_Key, _Value> *p = root;
	while (p->left != NIL)
		p = p->left;
	return p;
}

template <typename _Key, typename _Value, typename _Compare>
RBNode<_Key, _Value>* RBTree<_Key, _Value, _Compare>::maximum()
{
	if (root == NULL) return NULL;

	RBNode<_Key, _Value> *p = root;
	while (p->right != NIL)
		p = p->right;
	return p;
}

template <typename _Key, typename _Value, typename _Compare>
RBNode<_Key, _Value>* RBTree<_Key, _Value, _Compare>::find(_Key k)
{
	if (root == NULL) return NULL;

	RBNode<_Key, _Value> *p = root;
	while (p != NIL)
	{
		if (key_compare(k, p->key)) // k < p->key
			p = p->left;
		else if (key_compare(p->key, k)) // k > p->key
			p = p->right;
		else // k == p->key
			break;
	}
	return p != NIL ? p : NULL;
}

template <typename _Key, typename _Value, typename _Compare>
void RBTree<_Key, _Value, _Compare>::clear()
{
	if (node_count == 0) return;

	std::stack<RBNode<_Key, _Value>* > S;
	RBNode<_Key, _Value> *p = root, *q = NULL;
	while (p != NIL || !S.empty())
	{
		if (p != NIL)
		{
			S.push(p);
			p = p->left;
		}
		else
		{
			p = S.top();
			S.pop();
			q = p;
			p = p->right;
			delete q;
		}
	}
	
	node_count = 0;
	root = NULL;
}

template <typename _Key, typename _Value, typename _Compare>
RBNode<_Key, _Value>* RBTree<_Key, _Value, _Compare>::insert(_Key k, _Value v)
{
	if (root == NULL)
	{
		root = new RBNode<_Key, _Value>(k, v, NIL);
		root->color = Color::B;
		++node_count;
		return root;
	}

	RBNode<_Key, _Value> *p = root;
	std::vector<RBNode<_Key, _Value>* > S;
	S.reserve(32);
	while (p != NIL)
	{
		S.push_back(p);
		if (key_compare(k, p->key)) // k < p->key
			p = p->left;
		else if (key_compare(p->key, k)) // k > p->key
			p = p->right;
		else // k == p->key
			return NULL;
	}
	p = new RBNode<_Key, _Value>(k, v, NIL);
	RBNode<_Key, _Value> *q = S.back(), *r = p;
	if (q->key > p->key)
		q->left = p;
	else
		q->right = p;
	++node_count;

	_insertRebalance(p, S);

	return r;
}

template <typename _Key, typename _Value, typename _Compare>
bool RBTree<_Key, _Value, _Compare>::erase(_Key k)
{
	if (root == NULL)
		return false;
	if (node_count == 1)
	{
		if (root->key != k)
			return false;
		delete root;
		root = NULL;
		node_count = 0;
		return true;
	}

	RBNode<_Key, _Value> *p = root;
	std::vector<RBNode<_Key, _Value>* > S;
	S.reserve(32);
	while (p != NIL)
	{
		S.push_back(p);
		if (key_compare(k, p->key)) // k < p->key
			p = p->left;
		else if (key_compare(p->key, k)) // k > p->key
			p = p->right;
		else // k == p->key
			break;
	}
	if (p == NIL)
		return false;
	RBNode<_Key, _Value> *q = p, *r = q;
	if (p->left != NIL) // q is predecessor of p
	{
		q = p->left;
		S.push_back(q);
		while (q->right != NIL)
		{
			r = q;
			q = q->right;
			S.push_back(q);
		}
		if (q == r->left)
			r->left = q->left;
		else
			r->right = q->left;
	}
	else if (p->right != NIL) // q is successor of p
	{
		q = p->right;
		S.push_back(q);
		while (q->left != NIL)
		{
			r = q;
			q = q->left;
			S.push_back(q);
		}
		if (q == r->left)
			r->left = q->right;
		else
			r->right = q->right;
	}
	S.pop_back(); // let S.back() == parent of q
	if (q != p)
	{
		p->key = q->key;
		p->value = q->value;
	}
	else
	{
		if (q == S.back()->left)
			S.back()->left = NIL;
		else
			S.back()->right = NIL;
	}
	if (q->color == Color::B)
		_eraseRebalance(p->left != NIL ? q->left : q->right, S);

	delete q;
	--node_count;

	return true;
}

template <typename _Key, typename _Value, typename _Compare>
void RBTree<_Key, _Value, _Compare>::_insertRebalance(RBNode<_Key, _Value> *p, std::vector<RBNode<_Key, _Value>* >& S)
{
	int h = static_cast<int>(S.size()); // height of p
	while (h > 0 && S[h-1]->color == Color::R)
	{
		RBNode<_Key, _Value> *q = S[h-1], *r = S[h-2]; // q is parent of p, r is parent of q
		h -= 2; // now, S[h] is r
		if (q == r->left)
		{
			RBNode<_Key, _Value> *t = r->right; // t is uncle of p
			if (t->color == Color::R)
			{
				q->color = Color::B;
				t->color = Color::B;
				r->color = Color::R;
				p = r;
			}
			else
			{
				if (p == q->right)
				{
					if (q == r->left)
						r->left = leftRotate(&q);
					else
						r->right = leftRotate(&q);
					p->color = Color::B;
				}
				else
					q->color = Color::B;
				r->color = Color::R;
				if (h > 0) // r is not the root
				{
					if (r == S[h-1]->left)
						S[h-1]->left = rightRotate(&r);
					else
						S[h-1]->right = rightRotate(&r);
				}
				else // r is the root
					root = rightRotate(&r);
				break;
			}
		}
		else // (q == r->right)
		{
			RBNode<_Key, _Value> *t = r->left; // t is uncle of p
			if (t->color == Color::R)
			{
				q->color = Color::B;
				t->color = Color::B;
				r->color = Color::R;
				p = r;
			}
			else
			{
				if (p == q->left)
				{
					if (q == r->left)
						r->left = rightRotate(&q);
					else
						r->right = rightRotate(&q);
					p->color = Color::B;
				}
				else
					q->color = Color::B;
				r->color = Color::R;
				if (h > 0) // r is not the root
				{
					if (r == S[h-1]->left)
						S[h-1]->left = leftRotate(&r);
					else
						S[h-1]->right = leftRotate(&r);
				}
				else // r is the root
					root = leftRotate(&r);
				break;
			}
		}
	}
	root->color = Color::B;
}

template <typename _Key, typename _Value, typename _Compare>
void RBTree<_Key, _Value, _Compare>::_eraseRebalance(RBNode<_Key, _Value> *p, std::vector<RBNode<_Key, _Value>* >& S)
{
	int h = static_cast<int>(S.size()); // height of p
	while (p != root && p->color == Color::B)
	{
		RBNode<_Key, _Value> *q = S[h-1], *r = h > 1 ? S[h-2] : NULL; // q is parent of p, r is parent of q
		if (p == q->left)
		{
			RBNode<_Key, _Value> *w = q->right; // w is brother of p
			if (w->color == Color::R)
			{
				w->color = Color::B;
				q->color = Color::R;
				if (r != NULL)
				{
					if (q == r->left)
						r = r->left = leftRotate(&q);
					else
						r = r->right = leftRotate(&q);
				}
				else
					r = root = leftRotate(&q);
				S.back() = w;
				S.push_back(q);
				++h;
				w = q->right;
			}
			if (w->left->color == Color::B && w->right->color == Color::B)
			{
				w->color = Color::R;
				p = q;
				S.pop_back();
				--h;
			}
			else
			{
				if (w->right->color == Color::B)
				{
					w->left->color = Color::B;
					w->color = Color::R;
					q->right = rightRotate(&w);
					w = q->right;
				}
				w->color = q->color;
				q->color = Color::B;
				w->right->color = Color::B;
				if (r != NULL)
				{
					if (q == r->left)
						r->left = leftRotate(&q);
					else
						r->right = leftRotate(&q);
				}
				else
					root = leftRotate(&q);
				p = root;
			}
		}
		else
		{
			RBNode<_Key, _Value> *w = q->left; // w is brother of p
			if (w->color == Color::R)
			{
				w->color = Color::B;
				q->color = Color::R;
				if (r != NULL)
				{
					if (q == r->left)
						r = r->left = rightRotate(&q);
					else
						r = r->right = rightRotate(&q);
				}
				else
					r = root = rightRotate(&q);
				S.back() = w;
				S.push_back(q);
				++h;
				w = q->left;
			}
			if (w->left->color == Color::B && w->right->color == Color::B)
			{
				w->color = Color::R;
				p = q;
				S.pop_back();
				--h;
			}
			else
			{
				if (w->left->color == Color::B)
				{
					w->right->color = Color::B;
					w->color = Color::R;
					q->left = leftRotate(&w);
					w = q->left;
				}
				w->color = q->color;
				q->color = Color::B;
				w->left->color = Color::B;
				if (r != NULL)
				{
					if (q == r->left)
						r->left = rightRotate(&q);
					else
						r->right = rightRotate(&q);
				}
				else
					root = rightRotate(&q);
				p = root;
			}
		}
	}
	p->color = Color::B;
}

template <typename _Key, typename _Value, typename _Compare>
RBNode<_Key, _Value>* RBTree<_Key, _Value, _Compare>::leftRotate(RBNode<_Key, _Value> **p)
{
	RBNode<_Key, _Value> *q = (*p)->right;
	(*p)->right = q->left;
	q->left = *p;
	return q;
}

template <typename _Key, typename _Value, typename _Compare>
RBNode<_Key, _Value>* RBTree<_Key, _Value, _Compare>::rightRotate(RBNode<_Key, _Value> **p)
{
	RBNode<_Key, _Value> *q = (*p)->left;
	(*p)->left = q->right;
	q->right = *p;
	return q;
}

template <typename _Key, typename _Value, typename _Compare>
void RBTree<_Key, _Value, _Compare>::preOrderTraverse(std::vector<RBNode<_Key, _Value>* >& vec)
{
	if (node_count == 0) return;

	vec.reserve(node_count);

	std::stack<RBNode<_Key, _Value>* > S;
	RBNode<_Key, _Value> *p = root;
	while (p != NIL || !S.empty())
	{
		if (p != NIL)
		{
			vec.push_back(p);
			S.push(p);
			p = p->left;
		}
		else
		{
			p = S.top();
			S.pop();
			p = p->right;
		}
	}
}

template <typename _Key, typename _Value, typename _Compare>
void RBTree<_Key, _Value, _Compare>::inOrderTraverse(std::vector<RBNode<_Key, _Value>* >& vec)
{
	if (node_count == 0) return;

	vec.reserve(node_count);

	std::stack<RBNode<_Key, _Value>* > S;
	RBNode<_Key, _Value> *p = root;
	while (p != NIL || !S.empty())
	{
		if (p != NIL)
		{
			S.push(p);
			p = p->left;
		}
		else
		{
			p = S.top();
			S.pop();
			vec.push_back(p);
			p = p->right;
		}
	}
}

template <typename _Key, typename _Value, typename _Compare>
void RBTree<_Key, _Value, _Compare>::postOrderTraverse(std::vector<RBNode<_Key, _Value>* >& vec)
{
	if (node_count == 0) return;

	vec.reserve(node_count);

	std::stack<RBNode<_Key, _Value>* > S;
	RBNode<_Key, _Value> *prev = NULL, *cur = NULL;
	S.push(root);
	while (!S.empty())
	{
		cur = S.top();
		/* visit cur when it is a leaf node or its childs have been visited */
		if ( (cur->left == NIL && cur->right == NIL) || (prev != NULL && (prev == cur->left || prev == cur->right)) )
		{
			vec.push_back(cur);
			S.pop();
			prev = cur;
		}
		else
		{
			/* push() right child first, left child second, so left child comes first when pop() */
			if (cur->right != NIL)
				S.push(cur->right);
			if (cur->left != NIL)
				S.push(cur->left);
		}
	}
}

template <typename _Key, typename _Value, typename _Compare>
void RBTree<_Key, _Value, _Compare>::validateBlackHeight()
{
	int ph = -1, h = 0;
	fetchBlackHeight(root, ph, h);
}

template <typename _Key, typename _Value, typename _Compare>
void RBTree<_Key, _Value, _Compare>::fetchBlackHeight(RBNode<_Key, _Value> *node, int &ph, int& h)
{
	if (node != NULL)
	{
		h += node->color == Color::B ? 1 : 0;
		if (node->left == NIL && node->right == NIL)
		{
			if (ph == -1)
				ph = h;
			assert(h == ph);
		}
		fetchBlackHeight(node->left, ph, h);
		fetchBlackHeight(node->right, ph, h);
		h -= node->color == Color::B ? 1 : 0;
	}
}

#endif /* __RBTREE_H__ */

