#include "RBTree.h"

#include <time.h>
#include <iostream>
#include <map>
#include <algorithm> // random_shuffle

using std::cout;
using std::endl;

/**
 * @brief convenience class, which prints procedure executing time between Timer's construction and its destruction.
 */
class Timer
{
public:
	/** record constructing time in the constructor. */
	explicit Timer(const char *s) : _note(s) { _start_time = clock(); }
	~Timer() { cout << _note << elapsed() << "s." << endl; }
	/** calculate how much time elapsed since constructed. */
	inline double elapsed() { return static_cast<double>(clock() - _start_time) / CLOCKS_PER_SEC; }

private:
	Timer(const Timer&);
	Timer& operator=(const Timer&);

private:
	std::string _note;   ///< this string will be printed when timer elapsed.
	clock_t _start_time; ///< record the time when the timer is constructed.
};

namespace nonstd
{

/**
 * @brief A non-standard map made up of (key,value) pairs, which can be retrieved based on a key, in logarithmic time.
 * 
 * @tparam _K  Type of key objects.
 * @tparam _V  Type of mapped objects.
 * @tparam _C  Comparison function object type, defaults to less<_K>.
 */
template <typename _K, typename _V, typename _C = std::less<_K> >
class map
{
public:
	/** @name container typedefs for easy of use. */
	///@{
	typedef _K key_type;
	typedef _V data_type;
	typedef _V mapped_type;
	typedef std::pair<const _K, _V> value_type;
	typedef _C key_compare;

	typedef RBNode<_K, _V>* iterator;
	///@}

	/** @name constructors, destructor of nonstd::map. */
	///@{
	map() : _M_t() {}
	explicit map(const _C& __comp) : _M_t(__comp) {}
	map(const map& __x) : _M_t(__x._M_t) { }
	~map() { clear(); }
	///@}
	
	map& operator=(const map& __x) { _M_t = __x._M_t; return *this; }

	/** @name APIs of nonstd::map. */
	///@{
	bool empty() { return _M_t.empty(); }
	size_t size() { return _M_t.size(); }
	
	iterator begin() { return _M_t.minimum(); }
	iterator end() { return NULL; }

	iterator find(const key_type& __x) { return _M_t.find(__x); }
	size_t count(const key_type& __x) { return _M_t.find(__x) == end() ? 0 : 1; }

	iterator insert(const value_type& __x) { return _M_t.insert(__x.first, __x.second); }
	void erase(const key_type& __x) { _M_t.erase(__x); }

	void clear() { _M_t.clear(); }
	///@}

	void validate() { _M_t.validateBlackHeight(); }

private:
	RBTree<_K, _V, _C> _M_t; ///< Red-Black tree representing nonstd::map.
};

} // end namespace nonstd

#define DATA_SCALE    1000000
#define FAIL_RATIO    0.30

inline double dblrand() { return static_cast<double>(rand())/RAND_MAX + (static_cast<double>(rand())/RAND_MAX * (1.0/RAND_MAX)); }
inline int idxrand(int minIdx, int maxIdx) { return rand() % (maxIdx - minIdx) + minIdx; }

template <typename _Key, typename _Value>
void printTraverseResult(std::vector<RBNode<_Key, _Value>* >& v)
{
	for (size_t i = 0; i < v.size(); ++i)
		cout << v[i]->key << '(' << (v[i]->color == Color::R ? 'R' : 'B') << ") ";
	cout << endl;
}

void fillRBTree(RBTree<int, double>& rbTree)
{
	rbTree.insert(1, 3.23);
	rbTree.insert(2, 1.34);
	rbTree.insert(3, 8.36);
	rbTree.insert(4, 9.73);
	rbTree.insert(5, 7.18);
	rbTree.insert(6, 7.51);
	rbTree.insert(7, 8.13);
	rbTree.insert(16, 4.16);
	rbTree.insert(15, 2.86);
	rbTree.insert(14, 1.01);
	rbTree.insert(13, 5.69);
	rbTree.insert(12, 4.97);
	rbTree.insert(11, 9.61);
	rbTree.insert(10, 3.27);
	rbTree.insert(8, 7.12);
	rbTree.insert(9, 8.69);
	
	/*
	 * After fill, the structure of RB tree looks:
	 *                      4(B)
	 * 
	 *              /               \
	 *
	 *            2(B)                13(R)
	 *       /         \         /          \
	 *      1(B)        3(B)    8(B)        15(B)
	 *                       /     \     /      \
	 *                     6(R)   11(R) 14(B)  16(B)
	 *                     / \     / \
	 *                  5(B)7(B) 10(B)12(B)
	 *                           /
	 *                         9(R)
	 */
}

int main(int argc, char *argv[])
{
	cout << "Test RBTree methods: " << endl;

	RBTree<int, double> rbTree;
	fillRBTree(rbTree);

	cout << "before erase :\n";
	std::vector<RBNode<int, double>* > preOrderRB, inOrderRB, postOrderRB;
	rbTree.preOrderTraverse(preOrderRB);
	rbTree.inOrderTraverse(inOrderRB);
	rbTree.postOrderTraverse(postOrderRB);
	cout << "PreOrder: "; printTraverseResult(preOrderRB);
	cout << "InOrder: "; printTraverseResult(inOrderRB);
	cout << "PostOrder: "; printTraverseResult(postOrderRB);
	preOrderRB.clear(); inOrderRB.clear(); postOrderRB.clear();
	
	rbTree.erase(1);
	
	/*
	 * After erase 1, the structure of RB tree looks:
	 *                     13(B)
	 * 
	 *              /               \
	 *
	 *            8(R)                15(B)
	 *       /         \         /          \
	 *      4(B)       11(B)   14(B)        16(B)
	 *   /     \    /     \ 
	 * 2(B)  6(R)  10(B) 12(B)
	 *   \   / \   /
	 *   3R 5B 7B 9R
	 */
	cout << "after erase :\n";
	rbTree.preOrderTraverse(preOrderRB);
	rbTree.inOrderTraverse(inOrderRB);
	rbTree.postOrderTraverse(postOrderRB);
	cout << "PreOrder: "; printTraverseResult(preOrderRB);
	cout << "InOrder: "; printTraverseResult(inOrderRB);
	cout << "PostOrder: "; printTraverseResult(postOrderRB);
	
	rbTree.clear();
	
	/*
	cout << "performance evaluation of RBTree, compared with std::map which implemented by red-black tree.\n";
	std::map<int, double> std_map;
	nonstd::map<int, double> nonstd_map;

	unsigned int seed = 1;
	// srand(time(NULL));
	srand(seed);
	
	int i = 0;
	// generate random keys and values
	std::vector<int> keys;
	std::vector<double> values;
	keys.reserve(DATA_SCALE);
	values.reserve(DATA_SCALE);
	for (i = 0; i < DATA_SCALE; ++i)
	{
		keys.push_back(rand());
		values.push_back(dblrand());
	}
	
	// determine the order of elements' index in the operations of insert(), erase() and find()
	std::vector<int> insertOrder, eraseOrder, findOrder;
	insertOrder.reserve(2*DATA_SCALE);
	eraseOrder.reserve(DATA_SCALE);
	findOrder.reserve(DATA_SCALE);
	for (i = 0; i < DATA_SCALE; ++i) // 0-100%
	{
		insertOrder.push_back(i); // 100%
		if (i % 4 == 0)
			eraseOrder.push_back(i); // 25%
		else if (i % 4 == 1)
			findOrder.push_back(i); // 25%
	}
	for (i = 0; i < FAIL_RATIO*DATA_SCALE; ++i)
	{
		int idx = idxrand(0, DATA_SCALE);
		insertOrder.push_back(idx); // FAIL_RATIO * 100%
	}
	for (i = 0; i < FAIL_RATIO*DATA_SCALE; i += 2)
	{
		int idx = idxrand(0, DATA_SCALE);
		while (idx % 4 < 2)
			idx = idxrand(0, DATA_SCALE);
		eraseOrder.push_back(idx); // FAIL_RATIO * 50%
	}
	for (i = 0; i < FAIL_RATIO*DATA_SCALE; i += 2)
	{
		int idx = idxrand(0, DATA_SCALE);
		while (idx % 4 < 2)
			idx = idxrand(0, DATA_SCALE);
		findOrder.push_back(idx); // FAIL_RATIO * 50%
	}

	std::random_shuffle(insertOrder.begin(), insertOrder.end()); // using built-in random generator
	std::random_shuffle(eraseOrder.begin(), eraseOrder.end()); // using built-in random generator
	std::random_shuffle(findOrder.begin(), findOrder.end()); // using built-in random generator

	// execute operations of insert(), erase() and find() for both std::map and nonstd::map
	{
		Timer timer("insertion of std::map costs: ");
		for (i = 0; i < DATA_SCALE * (1+FAIL_RATIO); ++i)
			std_map.insert(std::pair<int, double>(keys[insertOrder[i]], values[insertOrder[i]]));
	}
	nonstd_map.validate();
	{
		Timer timer("insertion of nonstd::map costs: ");
		for (i = 0; i < DATA_SCALE * (1+FAIL_RATIO); ++i)
			nonstd_map.insert(std::pair<int, double>(keys[insertOrder[i]], values[insertOrder[i]]));
	}
	{
		Timer timer("erasure of std::map costs: ");
		for (i = 0; i < DATA_SCALE/2 * (1+FAIL_RATIO); ++i)
			std_map.erase(keys[eraseOrder[i]]);
		cout << "[erasure] std::map elements' count: " << std_map.size() << endl;
	}
	{
		Timer timer("erasure of nonstd::map costs: ");
		for (i = 0; i < DATA_SCALE/2 * (1+FAIL_RATIO); ++i)
			nonstd_map.erase(keys[eraseOrder[i]]);
		cout << "[erasure] nonstd::map elements' count: " << nonstd_map.size() << endl;
	}
	nonstd_map.validate();
	{
		Timer timer("finding of std::map costs: ");
		int existCount = 0;
		for (i = 0; i < DATA_SCALE/2 * (1+FAIL_RATIO); ++i)
			if (std_map.find(keys[findOrder[i]]) != std_map.end())
				++existCount;
		cout << "std::map::find() exist elements' count: " << existCount << endl;
	}
	{
		Timer timer("finding of nonstd::map costs: ");
		int existCount = 0;
		for (i = 0; i < DATA_SCALE/2 * (1+FAIL_RATIO); ++i)
			if (nonstd_map.find(keys[findOrder[i]]) != nonstd_map.end())
				++existCount;
		cout << "nonstd::map::find() exist elements' count: " << existCount << endl;
	}

	std_map.clear();
	nonstd_map.clear();

	cout << "From comparsion, we can draw a conclusion that the performance of nonstd::map is slightly worse than std::map." << endl;
	*/
	return 0;
}

